#include <Wire.h>
#include <Tiny4kOLED.h> //OLED library

#define SDA_PIN PB0
#define SCL_PIN PB2
#define LED_PIN PB3
#define I2C_DEV_ADDR 0x09
#define MSG_BUFFER_SIZE  (20)

//OLED variables
char msg[MSG_BUFFER_SIZE];
bool initialized = false;

void blink_led(){
  digitalWrite(LED_PIN, HIGH);
  delay(200);
  digitalWrite(LED_PIN, LOW);
  delay(200);
}
void blink_led_n(int n){
  for(int i = 0; i < n; ++i){
    blink_led();
  }
}

void initial_display()
{
  if (not initialized)
  {
    oled.begin();
    oled.setFont(FONT8X16);
    oled.clear(); // Clear the memory before turning on the display
    oled.on();    // Turn on the display
    // Switch the half of RAM that we are writing to, to be the half that is non currently displayed
    oled.switchRenderFrame();

    //initial message
    oled.clear();
    oled.setCursor(0, 5);
    oled.print(F("OLED ready!"));
    oled.switchFrame();
    delay(2000);
    initialized = true;
  }
}

void update_oled(int distance)
{ 
  // Clear the half of memory not currently being displayed.
  oled.clear();
  oled.setCursor(0, 1);
  oled.print(F("dist: "));
  oled.print(distance);
  oled.print(F("mm"));
  oled.switchFrame();
}

void setup() {
  pinMode(LED_PIN, OUTPUT);
  Wire.begin();        // join i2c bus (address optional for master)
  //OLED
  initial_display();
  blink_led_n(3);
  delay(1000);
}

void loop() {
  Wire.requestFrom(I2C_DEV_ADDR, 1);    // request 1 bytes from slave device #9

  while (Wire.available()) { // slave may send less than requested
    int i2c_value = Wire.read(); // receive a distance as integer
    update_oled(i2c_value);
    blink_led();
  }

  delay(500);
}
