#include "mqttFabAcademy.h" //wifi library defined for this assignment

//declaration and initialization of variables

//declaration of MQTT server variables
const char* mqtt_server = "192.168.4.1";
const int mqtt_port = 1883;
WiFiClient espClient;
const char* mqttServerStatus = "mqttServerStatus";
PubSubClient MQTTclient(espClient);

String msgReceived = "";  //message received from MQTT
String msgSend;           //message to be sent by MQTT
char* mqttSensorTopic = "mqttSensorListener";
char* mqttOLEDTopic = "mqttOLEDListener";

void mqttMessageCallback(char* topic, byte* payload, unsigned int length) {
  msgReceived = "";
  for(int i=0; i<length;++i){
    int inChar = (char)payload[i];
    if(isDigit(inChar)){
      msgReceived += (char)inChar;
    }
  }
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("]: ");
  Serial.println(msgReceived);
}

void setupMqttClient() {
  // to call in the setup of the arduino
  MQTTclient.setServer(mqtt_server, mqtt_port);
  MQTTclient.setCallback(mqttMessageCallback);
}

void mqttSubscribeTopic(char* topic){
  MQTTclient.subscribe(topic);
  Serial.print("MQTT subscribed to: ");
  Serial.println(topic);
}

void mqttPublishMessage(const char* topic, const char* payload) {
  Serial.print("Publishing message: ");
  Serial.println(payload);
  Serial.print("+Topic: ");
  Serial.println(topic);
  // API to publish to MQTT server. 
  MQTTclient.publish(topic, payload);

  Serial.println("MQTT Message sent.");
}

void mqttReconnect() {
  // Loop until MQTT is reconnected
  while (!MQTTclient.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Create a random client ID
    String clientId = "WemosSensorClient-";
    clientId += String(random(0xffff), HEX);
    Serial.printf("\n+clientId: %s", clientId);
    // Attempt to connect
    if (MQTTclient.connect(clientId.c_str())) {
      Serial.println("connected");
    } else {
      Serial.print("failed, rc=");
      Serial.print(MQTTclient.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

int handleMQTTClient() {
  int reconn = 0;
  // to call in the loop of the arduino
  if (!MQTTclient.connected()) {
    // LED on to  indicate connection problem
    digitalWrite(LED_BUILTIN, LOW);
    mqttReconnect();
    reconn = 1;
  } else {
    // LED off all is ok
    digitalWrite(LED_BUILTIN, HIGH);
  }
  MQTTclient.loop();
  return reconn;
}
