#include "mqttFabAcademy.h" //MQTT library defined for this assignment
#include <Wire.h>   //I2C library

#define I2C_DEV_ADDR 0x08

void setup() {
  Serial.begin(115200);           // start serial for output
  Serial.println("init setup...");
  
  setup_wifi();
  
  //MQTT
  setupMqttClient();
  mqttReconnect();
  mqttSubscribeTopic(mqttSensorTopic);
  mqttPublishMessage(mqttServerStatus, "Distance Sensor Connected.");
  Serial.println("MQTT ready.");
  
  //setup I2C communication
  Wire.begin((uint8_t)I2C_DEV_ADDR);  // join i2c bus with address #8 
  Wire.onReceive(receiveEvent);       // register receive event
  
  Serial.println("Sensor slave receiver ready.");
}

void loop() {
  //handle MQTT messages
  if(handleMQTTClient()){
    //reconnected, subscribe again
    mqttSubscribeTopic(mqttSensorTopic);
    // inform server wemos is re-connected
    mqttPublishMessage(mqttServerStatus, "Distance Sensor RE-Connected.");
  }
  delay(100);
}

// function that executes whenever data is received from master
// this function is registered as an event, see setup()
void receiveEvent(int howMany) {
  Serial.printf("onReceive[%d]: ", howMany);
  int distance = Wire.read();
  Serial.print("distance: ");
  Serial.println(distance);         // print distance
  
  //send distance over MQTT
  String sensor_msg = String(distance);
  int str_len = sensor_msg.length() + 1; 
  // Prepare the character array (the buffer) 
  char char_array[str_len];
  // Copy it over 
  sensor_msg.toCharArray(char_array, str_len);

  Serial.print("Publish message: ");
  Serial.println(char_array);
  mqttPublishMessage(mqttOLEDTopic, char_array);
}
